
#pragma once


#include <RobotAPI/components/units/RobotUnit/NJointControllers/NJointController.h>
#include <VirtualRobot/Robot.h>
#include <RobotAPI/components/units/RobotUnit/RobotUnit.h>
#include <RobotAPI/components/units/RobotUnit/ControlTargets/ControlTarget1DoFActuator.h>
#include <RobotAPI/components/units/RobotUnit/SensorValues/SensorValue1DoFActuator.h>
#include <dmp/representation/dmp/umidmp.h>
#include <ArmarXCore/core/services/tasks/RunningTask.h>
#include <RobotAPI/interface/units/RobotUnit/NJointJointSpaceDMPController.h>

namespace armarx
{

    TYPEDEF_PTRS_HANDLE(NJointJSDMPController);

    TYPEDEF_PTRS_HANDLE(NJointJSDMPControllerControlData);
    class NJointJSDMPControllerControlData
    {
    public:
        std::vector<double> targetJointVels;
    };


    //    class SimplePID
    //    {
    //    public:
    //        float Kp = 0, Kd = 0;
    //        float lastError = 0;
    //        float update(float dt, float error);
    //    };

    /**
     * @brief The NJointJSDMPController class
     * @ingroup Library-RobotUnit-NJointControllers
     */
    class NJointJSDMPController :
        public NJointControllerWithTripleBuffer<NJointJSDMPControllerControlData>,
        public NJointJointSpaceDMPControllerInterface
    {
    public:
        using ConfigPtrT = NJointJointSpaceDMPControllerConfigPtr;
        NJointJSDMPController(const RobotUnitPtr& robotUnit, const NJointControllerConfigPtr& config, const VirtualRobot::RobotPtr&);

        // NJointControllerInterface interface
        std::string getClassName(const Ice::Current&) const override;

        // NJointController interface
        void rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) override;

        bool isFinished(const Ice::Current&) override
        {
            return finished;
        }

        void learnDMPFromFiles(const Ice::StringSeq& fileNames, const Ice::Current&) override;
        void setTemporalFactor(double tau, const Ice::Current&) override;

        void runDMP(const Ice::DoubleSeq&  goals, double tau, const Ice::Current&) override;

        void showMessages(const Ice::Current&) override;

    protected:
        void rtPreActivateController() override;
        void rtPostDeactivateController() override;

        virtual void onPublish(const SensorAndControl&, const DebugDrawerInterfacePrx&, const DebugObserverInterfacePrx&) override;
    private:

        struct DebugBufferData
        {
            StringFloatDictionary latestTargetVelocities;
            double currentCanVal;
            double mpcFactor;
        };
        TripleBuffer<DebugBufferData> debugOutputData;


        struct NJointJSDMPControllerSensorData
        {
            double currentTime;
            double deltaT;
            DMP::Vec<DMP::DMPState> currentState;
        };
        TripleBuffer<NJointJSDMPControllerSensorData> controllerSensorData;


        std::map<std::string, const SensorValue1DoFActuatorTorque*> torqueSensors;
        std::map<std::string, const SensorValue1DoFGravityTorque*> gravityTorqueSensors;
        std::map<std::string, const SensorValue1DoFActuatorPosition*> positionSensors;
        std::map<std::string, const SensorValue1DoFActuatorVelocity*> velocitySensors;
        std::map<std::string, ControlTarget1DoFActuatorVelocity*> targets;

        IceUtil::Time last;

        std::vector<double> goals;
        DMP::UMIDMPPtr dmpPtr;
        bool DMPAsForwardControl;
        double timeDuration;
        DMP::Vec<DMP::DMPState> currentState;
        double canVal;
        double deltaT;

        double tau;
        double finished;

        // phaseStop parameters
        double phaseL;
        double phaseK;
        double phaseDist0;
        double phaseDist1;
        double phaseKp;

        bool isDisturbance;
        std::vector<std::string> dimNames;
        DMP::DVec targetState;
        std::vector<double> targetVels;

        mutable MutexType controllerMutex;
        PeriodicTask<NJointJSDMPController>::pointer_type controllerTask;
        // ManagedIceObject interface
    protected:
        void controllerRun();
        void onInitComponent() override;
        void onDisconnectComponent() override;

    };

} // namespace armarx

