/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/RobotNodeSet.h>
#include <VirtualRobot/IK/DifferentialIK.h>
#include <VirtualRobot/Robot.h>
#include <Eigen/Dense>

namespace armarx
{
    class CartesianPositionController;
    using CartesianPositionControllerPtr = std::shared_ptr<CartesianPositionController>;

    // This is a P-controller. In: Target pose, out Cartesian velocity.
    class CartesianPositionController
    {
    public:
        CartesianPositionController(const VirtualRobot::RobotNodePtr& tcp);

        Eigen::VectorXf calculate(const Eigen::Matrix4f& targetPose, VirtualRobot::IKSolver::CartesianSelection mode) const;

        float getPositionError(const Eigen::Matrix4f& targetPose) const;
        float getOrientationError(const Eigen::Matrix4f& targetPose) const;
        Eigen::Vector3f getPositionDiff(const Eigen::Matrix4f& targetPose) const;
        Eigen::Vector3f getOrientationDiff(const Eigen::Matrix4f& targetPose) const;

        float KpPos = 1;
        float KpOri = 1;
        float maxPosVel = -1;
        float maxOriVel = -1;


    private:
        VirtualRobot::RobotNodePtr tcp;
    };
}

